unit unitmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Unix, FileUtil, TAGraph, TASeries, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, unitlib;

Const RET_OK    = 0;

      _128 = 128;
      _256 = _128 * 2;
      _512 = _256 * 2;
      _1K = _512 * 2;
      _2K = _1K * 2;
      _4K = _2K * 2;
      _10K = _2K * 5;
      _100K = _10K * 10;
      _500k = _100K * 5;
      _1M = _500K * 2;
      _2M = _1M * 2;
      _3M = _1M * 3;
      _4M = _1M * 4;
      _5M = _1M * 5;
      _10M = _5M * 2;
      _20M = _10M * 2;
      _50M = _10M * 5;
      _100M = _10M * 10;
      _1G = _100M * 10;

      MAX_BUFFER_SIZE = 8192;

type

  TCamData = record
    case integer of
      0: (ucBuf : array[0..(MAX_BUFFER_SIZE-1)] of byte);
      1: (wBuf : array[0..((MAX_BUFFER_SIZE DIV 2) - 1)] of word);
    end;

  { TThCamReader }

  TThCamReader = class(TThread)
    private
      FNumRead : dword;
      FFPS    : dword;
      FMem    : dword;
      F16bit  : boolean;
      FPixelCnt : WORD;
      FCamBuf  : TCamData;
      procedure ShowStatus;
      Function GetUSec : UInt64;
    protected
      procedure Execute; override;
    public
    end;

  { TForm1 }

  TForm1 = class(TForm)
    btn_trigtime: TButton;
    btn_enum: TButton;
    btn_exptime: TButton;
    btn_delay: TButton;
    btn_open: TButton;
    btn_savesettings: TButton;
    btn_setcfg: TButton;
    btn_start: TButton;
    btn_info: TButton;
    cb_168BitMode: TComboBox;
    cb_devices: TComboBox;
    cb_fullscale: TComboBox;
    cb_imagenum: TComboBox;
    cb_mode: TComboBox;
    Chart1: TChart;
    Chart1LineSeries1: TLineSeries;
    ed_trigtime: TEdit;
    ed_exptime: TEdit;
    ed_delay: TEdit;
    ed_numframes: TEdit;
    GroupBox6: TGroupBox;
    Label11: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label17: TLabel;
    Label26: TLabel;
    lb_gain: TLabel;
    lb_offset: TLabel;
    PageControl1: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    StatusBar1: TStatusBar;
    TabSheet5: TTabSheet;
    tb_gain: TTrackBar;
    tb_offset: TTrackBar;
    ud_numframes: TUpDown;
    procedure btn_delayClick(Sender: TObject);
    procedure btn_enumClick(Sender: TObject);
    procedure btn_exptimeClick(Sender: TObject);
    procedure btn_infoClick(Sender: TObject);
    procedure btn_savesettingsClick(Sender: TObject);
    procedure btn_setcfgClick(Sender: TObject);
    procedure btn_startClick(Sender: TObject);
    procedure btn_trigtimeClick(Sender: TObject);
    procedure cb_devicesChange(Sender: TObject);
    procedure cb_fullscaleChange(Sender: TObject);
    procedure cb_modeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tb_gainChange(Sender: TObject);
    procedure tb_offsetChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    procedure to_statusbar(stxt : string);
    procedure open_device(idx : integer);
    procedure closeusbdevice;
    procedure readsettings;
    procedure StartCamThread(APxlCnt : WORD; A16Mode : Boolean);
    procedure StopCamThread;
    function FormatSize(ASize : Integer) : String;
    procedure initchart(wpxlcnt : word; a16bit : boolean);
  end;

var
  Form1        : TForm1;
  CamThread    : TThCamReader;
  PipeSize     : Integer;
  Packetlength : Integer;
  wSensorType  : WORD;
  wPixelCount  : WORD;

  PACKETLENGTH_MULTIPLIER : Byte;
  MODE_16BIT : Boolean;

implementation
uses unitinfo;

{$R *.lfm}

{ TThCamReader }

procedure TThCamReader.ShowStatus;
var i : integer;
begin
  form1.Chart1LineSeries1.BeginUpdate;

  if F16bit then
  begin
    for i := 0 to (FNumRead div 2)-1 do
      form1.Chart1LineSeries1.SetYValue(i,FCamBuf.wBuf[i]);
  end else
  begin
    for i := 0 to FNumRead -1 do
      form1.Chart1LineSeries1.SetYValue(i,FCamBuf.ucBuf[i]);
  end;
  form1.Chart1LineSeries1.EndUpdate;

  Form1.Statusbar1.Panels[1].Text := 'FPS: '+IntToStr(FFPS);
  Form1.Statusbar1.Panels[2].Text := 'MEM: '+Form1.FormatSize(FMem);
  if form1.cb_mode.ItemIndex = 0 then form1.btn_start.Caption := 'Start';
end;

function TThCamReader.GetUSec: UInt64;
Var TZ : TimeVal;
Begin
  fpGetTimeOfDay(@TZ,Nil);
  Result := TZ.tv_usec + TZ.tv_sec*1000000;
End;

procedure TThCamReader.Execute;
var mSecStart : UInt64;
    imagesize : DWORD;
begin
  mSecStart := GetUSec;

  if F16bit then imagesize := (FPixelCnt * 2)
  else imagesize := FPixelCnt;

  while (not Terminated) do
  begin
    FMem := ls_waitforpipecount(imagesize,100);
    if FMem >= imagesize then
    begin
      FNumRead := ls_getpipe(@FCamBuf.ucBuf[0],imagesize);
      FFPS := (ls_getfps DIV imagesize);
      if FNumRead > 0 then
      begin
        FMem := FMem - FNumRead;
        if (GetUSec - mSecStart) >= 100000 then //TAChart component is too slow
        begin                                   //update chart & info every 100ms
          mSecStart := GetUSec;
          Synchronize(@ShowStatus);
        end;
      end;
    end;
  end;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF cpu64}
    Caption := 'USB Line Camera 8M (64Bit) - Lazarus Example';
  {$ELSE}
    Caption := 'USB Line Camera 8M (32Bit) - Lazarus Example';
  {$ENDIF}

  PACKETLENGTH_MULTIPLIER := 1;
  PipeSize     := _1M;
  wPixelCount  := _2K;
  Packetlength := (wPixelCount * 2) * PACKETLENGTH_MULTIPLIER;
  wSensorType  := 0;
  MODE_16BIT   := true;

  ls_initialize(PipeSize,Packetlength);
  CamThread := nil;
  initchart(wPixelCount,MODE_16BIT);
end;

procedure TForm1.to_statusbar(stxt: string);
begin
  StatusBar1.Panels[0].Text := stxt;
end;

procedure TForm1.StartCamThread(APxlCnt : WORD; A16Mode : Boolean);
begin
  if CamThread = nil then
  begin
    CamThread := TThCamReader.Create(true);
    CamThread.FPixelCnt := APxlCnt;
    CamThread.F16bit:= A16Mode;
    CamThread.Start;
  end;
end;

procedure TForm1.StopCamThread;
begin
  if CamThread <> nil then
  begin
    CamThread.Terminate;
    CamThread.WaitFor;
    FreeAndNil(CamThread);
  end;
end;

function TForm1.FormatSize(ASize: Integer): String;
begin
  if ASize > _1G then
  begin
    result := Format('%f',[ASize/_1G]) +' GB';
  end else if ASize > _1M then
  begin
    result := Format('%f',[ASize/_1M]) +' MB';
  end else if ASize > _1K then
  begin
    result := Format('%f',[ASize/_1K]) +' KB';
  end else result := IntToStr(ASize) +' Bytes';
end;

procedure TForm1.initchart(wpxlcnt: word; a16bit: boolean);
var i : integer;
begin
  Chart1LineSeries1.Clear;
  Chart1.Extent.XMax := wpxlcnt -1;
  if a16Bit then Chart1.Extent.YMax := 65535
  else Chart1.Extent.YMax := 255;

  for i := 0 to wPxlcnt-1 do
      Chart1LineSeries1.AddXY(i,0);
end;

procedure TForm1.open_device(idx : integer);
var ierr : integer;
    tmp_packetlength : Integer = 0;
    tmp_pixel : word = 0;
    tmp_sensor : word = 0;
    uccfg1 : byte = 0;
begin
  if (cb_devices.Items.Count = 0) or (idx < 0) then exit;

  if ls_currentdeviceindex > -1 then
  begin
    closeusbdevice;
  end else
  begin
    ierr := ls_opendevicebyindex(cb_devices.ItemIndex);
    if  ierr = RET_OK then
    begin
      ls_setstate(0,1000);

      if ls_getcfg1(uccfg1,1000) = RET_OK then
        MODE_16BIT := ((uccfg1 and $02) = $02);

      if ls_getsensortype(tmp_sensor,tmp_pixel,1000) = RET_OK then
        wpixelcount := tmp_pixel;

      initchart(wpixelcount,MODE_16BIT);

      if ls_getpacketlength(tmp_packetlength,1000) = RET_OK then
      begin
        if tmp_packetlength <> (PacketLength DIV PACKETLENGTH_MULTIPLIER) then
        begin
          closeusbdevice;
          Packetlength := tmp_packetlength * PACKETLENGTH_MULTIPLIER;
          if ls_setpacketlength(Packetlength) = RET_OK then
            ls_opendevicebyindex(idx);
        end;
      end;
      if ls_currentdeviceindex >= 0 then
      begin
        readsettings;
        StartCamThread(wPixelCount,MODE_16BIT);
        btn_open.Caption := 'Close';
      end;
    end;
    to_statusbar('OPEN DEVICE: '+ls_geterrorstring(ierr));
  end;
end;

procedure TForm1.closeusbdevice;
begin
  StopCamThread;
  ls_closedevice;
  //WaitForCCDThread;
  btn_open.Caption := 'Open';
  to_statusbar('USB DEVICE CLOSED');
end;

procedure TForm1.btn_enumClick(Sender: TObject);
var cnt, i : integer;
begin
  cb_devices.Clear;
  StopCamThread;
  btn_open.Caption := 'Open';
  cnt := ls_enumdevices;
  if cnt > 0 then
  begin
    to_statusbar(IntToStr(cnt)+' DEVICE(S) FOUND');
    for i := 0 to cnt -1 do
    begin
      cb_devices.Items.Add(ls_getproductname(i)+' '+ls_getserialnumber(i));
    end;
    cb_devices.ItemIndex:= 0;
    open_device(cb_devices.ItemIndex);
  end else to_statusbar('No DEVICES FOUND');
end;

procedure TForm1.btn_delayClick(Sender: TObject);
var ierr : integer;
    dwdelay : DWORD;
begin
  dwdelay := StrToIntDef(ed_delay.Text,0);
  ed_delay.Text := IntToStr(dwdelay);
  ierr := ls_setextdelay(dwdelay,1000);
  to_statusbar('SET DELAY: '+ls_geterrorstring(ierr));

  ierr := ls_getextdelay(dwdelay,1000);
  if ierr = RET_OK then
    ed_delay.Text := IntToStr(dwdelay);
end;

procedure TForm1.cb_devicesChange(Sender: TObject);
begin
  open_device(cb_devices.ItemIndex);
end;

procedure TForm1.btn_startClick(Sender: TObject);
var ierr : integer;
    ucstate : byte = 0;
begin
  ierr := ls_getstate(ucstate,1000);
  if ierr <> 0 then
  begin
    to_statusbar('GET STATE: '+ls_geterrorstring(ierr));
    exit;
  end;
  ucstate := (ucstate xor 1) and 1;
  ierr := ls_setstate(ucstate,1000);
  if ierr = 0 then
  begin
    if ucstate = 1 then btn_start.caption := 'Stop'
    else btn_start.caption := 'Start';
  end else to_statusbar('SET STATE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_trigtimeClick(Sender: TObject);
var ierr : integer;
    dwtrigtime : DWORD;
begin
  dwtrigtime := StrToIntDef(ed_trigtime.Text,0);
  ed_trigtime.Text := IntToStr(dwtrigtime);
  ierr := ls_setsofttrigtime(dwtrigtime,1000);
  to_statusbar('SET SOFT TRIG TIME: '+ls_geterrorstring(ierr));

  ierr := ls_getsofttrigtime(dwtrigtime,1000);
  if ierr = RET_OK then
    ed_trigtime.Text := IntToStr(dwtrigtime);
end;

procedure TForm1.btn_exptimeClick(Sender: TObject);
var ierr : integer;
    dwexptime : DWORD;
begin
  dwexptime := StrToIntDef(ed_exptime.Text,10000);
  ed_exptime.Text := IntToStr(dwexptime);
  ierr := ls_setinttime(dwexptime,1000);
  to_statusbar('SET INT TIME: '+ls_geterrorstring(ierr));

  ierr := ls_getinttime(dwexptime,1000);
  if ierr = RET_OK then
    ed_exptime.Text := IntToStr(dwexptime);
end;

procedure TForm1.cb_fullscaleChange(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_setadcconfig(cb_fullscale.ItemIndex shl 7,1000);
  to_statusbar('SET FULL SCALE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.cb_modeChange(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_setmode(cb_mode.ItemIndex,1000);
  if ierr = RET_OK then
  begin
    if cb_mode.ItemIndex = 0 then
      btn_start.Caption := 'Start';
  end;
  to_statusbar('SET MODE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.tb_gainChange(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_setadcpga1(tb_gain.Position,1000);
  lb_gain.caption := Format('%.4f V/V',[6 / (1+(5*((63-tb_gain.Position)/63)))]);
  to_statusbar('SET GAIN: '+ls_geterrorstring(ierr));
end;

procedure TForm1.tb_offsetChange(Sender: TObject);
var fNewValue : Single;
    wValue : word;
    Sign   : word;
    ierr  : integer;
begin
  fNewValue := tb_offset.Position * 1.2;
  if (fNewValue < -300) then fNewValue := -300;
  if (fNewValue > 300) then fNewValue := 300;

  wvalue := abs(tb_offset.Position);
  if tb_offset.Position < 0 then sign := $100
  else sign := 0;
  wvalue := wvalue or sign;
  ierr := ls_setadcoffset1(wvalue,1000);

  lb_offset.caption := Format('%.1f mV',[fNewValue]);
  to_statusbar('SET OFFSET: '+ls_geterrorstring(ierr));
end;

procedure TForm1.readsettings;
Var ierr : integer;
    wValue : WORD = 0;
    fValue : Single;
    iValue : Integer;
    dwValue : DWORD = 0;
    ucValue : Byte = 0;
    bcdversion : word;
begin
  bcdversion := ls_getfwversion(ls_currentdeviceindex);

  if (bcdversion >= FM_01V20) then
  begin
    ed_trigtime.enabled := true;
    btn_trigtime.enabled := true;
    ierr := ls_getsofttrigtime(dwvalue,1000);
    if ierr = RET_OK then
    begin
      ed_trigtime.Text := IntToStr(dwvalue);
    end;
  end else
  begin
    ed_trigtime.enabled := false;
    btn_trigtime.enabled := false;
  end;

  ierr := ls_getadcconfig(wValue,1000);
  if ierr = RET_OK then
  begin
    cb_fullscale.OnChange := nil;
    cb_fullscale.ItemIndex := (wvalue shr 7) and $01;
    cb_fullscale.OnChange := @cb_fullscaleChange;
  end;

  ierr := ls_getadcpga1(wValue,1000);
  if ierr = RET_OK then
  begin
    tb_gain.OnChange := nil;
    fValue := 6 / (1+(5*((63-wValue)/63)));
    if fValue < 1 then fValue := 1;
    if fValue > 6 then fValue := 6;
    tb_gain.Position  := wValue;
    lb_gain.caption := Format('%.4f V/V',[fValue]);
    tb_gain.OnChange := @tb_gainChange;
  end;

  ierr := ls_getadcoffset1(wValue,1000);
  if ierr = RET_OK then
  begin
    tb_offset.OnChange := nil;

    ivalue := wvalue and $FF;
    if ((wvalue and $100) = $100) then ivalue := ivalue * (-1);

    fvalue := (ivalue * 1.2);
    if fvalue < -300 then fvalue := -300;
    if fvalue > 300 then fvalue := 300;
    tb_offset.position := ivalue;
    lb_offset.caption := Format('%.1f mV',[fValue]);
    tb_offset.OnChange := @tb_offsetChange;
  end;

  ierr := ls_getinttime(dwvalue,1000);
  if ierr = RET_OK then
    ed_exptime.Text := IntToStr(dwvalue);

  ierr := ls_getextdelay(dwValue,1000);
  if ierr = RET_OK then
    ed_delay.Text := IntToStr(dwValue);

  ierr := ls_getstate(ucValue,1000);
  if ierr = RET_OK then
  begin
    if ucValue = 0 then btn_start.caption := 'Start'
    else btn_start.caption := 'Stop';
  end;

  ierr := ls_getmode(ucValue,1000);
  if ierr = RET_OK then
  begin
    cb_mode.OnChange := nil;
    cb_mode.Items.Clear;
    cb_mode.Items.Add('One Shot');
    cb_mode.Items.Add('Ext. Trigger ');
    cb_mode.Items.Add('Free Running');
    cb_mode.Items.Add('Ext. Exp. Ctrl ');

    if (bcdversion >= FM_01V20) then
      cb_mode.Items.Add('Soft Triggger');

    cb_mode.ItemIndex := ucValue;
    cb_mode.OnChange := @cb_modeChange;
  end;

  ierr := ls_getcfg1(ucValue,1000);
  if ierr = RET_OK then
  begin
    cb_imagenum.ItemIndex := (ucValue and $01);
    cb_168BitMode.ItemIndex := (ucValue and $02) shr 1;
    ud_numframes.Position := ((ucValue and $7C) shr 2)+1;
  end;
end;

procedure TForm1.btn_savesettingsClick(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_savesettings(1000);
  to_statusbar('SAVE SETTINGS: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_setcfgClick(Sender: TObject);
var ierr : integer;
    uccfg1 : byte;
begin
  uccfg1 := ((ud_numframes.Position-1) and $1F);
  uccfg1 := (uccfg1 shl 1) or (cb_168BitMode.ItemIndex and $01);
  uccfg1 := (uccfg1 shl 1) or Byte(cb_imagenum.ItemIndex and $01);
  ierr := ls_setcfg1(uccfg1,1000);
  if ierr = RET_OK then
  begin
    Showmessage('Changes takes effect after reboot - Power OFF/ON');
  end;
  to_statusbar('SET CFG1: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_infoClick(Sender: TObject);
begin
  fminfo.devcount := cb_devices.Items.Count;
  fminfo.devindex := cb_devices.ItemIndex;
  fminfo.showmodal;
end;


end.

